local carPh = ac.accessCarPhysics()

--Setup--
local mainPressureK = 5.5
local brakePotencyFront = 500
local brakePotencyRear = 320
local staticFrictionK = 0.1
local brakeLag = 0.1
local brakeCoolingK = 0.035
local brakeCoolingAirflowK = 0.03
local brakeTempLut = ac.DataLUT11.parse('0=0.92|110=0.97|250=1|450=0.95|600=0.87')
local handBrakePressureK = 5
local handBrakeLag = 0.1
----

local function brakes20(i)
	local wheelPh = carPh.wheels[i]

	local brakeTemp = sim.ambientTemperature
	local brakeForce = 0
	local handBrakeForce = 0

	brakeTempLut.useCubicInterpolation = true
	brakeTempLut.extrapolate = true

    local function reset()
        brakeTemp = sim.ambientTemperature
    end

    local function update(dt)
		local brakePowerMult = ac.getSetupValue('BRAKE_POWER_MULT') or 1
		local mainPressure = mainPressureK * brakePowerMult.value
		local brakePressureK = (mainPressure * car.brakeBias)
		local handBrakePressure = 0
		local brakeTempK = math.saturate(brakeTempLut:get(brakeTemp))
		local lockedBrakes = wheelPh.angularSpeed < 0.1 and 1 or 0
		local brakePressure = carPh.brake * brakePressureK * (1 + staticFrictionK * lockedBrakes)
		local velLength = wheelPh.velocity:length()
		local velocityK = velLength / (1 + velLength)
		local brakePotency = brakePotencyFront
		if i > 1 then
			brakePotency = brakePotencyRear
			brakePressureK = mainPressureK - brakePressureK
			handBrakePressure = math.max(carPh.handbrake * handBrakePressureK - brakePressure, 0)
			handBrakeForce = handBrakeForce + (brakePotency * handBrakePressure * brakeTempK - handBrakeForce) * handBrakeLag
		end

		brakeForce = brakeForce + (brakePotency * brakePressure * brakeTempK - brakeForce) * brakeLag
		brakeTemp = brakeTemp + (brakePressure + handBrakePressure) * wheelPh.angularSpeed * dt
		brakeTemp = brakeTemp - brakeCoolingK * (brakeTemp - sim.ambientTemperature) * (1 + (wheelPh.angularSpeed * (1 - velocityK) + velLength) * brakeCoolingAirflowK) * dt

		ac.overrideBrakesTorque(i, brakeForce, handBrakeForce, 0)

    end

    return { update = update, reset = reset }
end

local wheels = {brakes20(0), brakes20(1), brakes20(2), brakes20(3)}

return {
    update = function (dt)
        for i = 1, 4 do
            wheels[i].update(dt)
        end
    end,
    reset = function ()
        for i = 1, 4 do
            wheels[i].reset()
        end
    end
}