local carPh = ac.accessCarPhysics()
local trottleCurve = ac.DataLUT11.carData(car.index, 'throttle.lut')

local minBoost = 0.7
local maxBoost = 1.5
local turboLag = 0.998
local antilag = 0.4
local antilagTime = 0.002

local antilagCounter = 0
local finalBoost = 0

local rpmLut = ac.DataLUT11.parse('1=-0.3|2=0|4=1.3|6=1.7|8=1.4')
rpmLut.useCubicInterpolation = true
rpmLut.extrapolate = true

local function reset()
	finalBoost = 0
end

local function update(dt)
	local boostController = minBoost + (maxBoost - minBoost) * ac.getTurboUserWastegate(0)
	local actualTrottle = trottleCurve:get(carPh.gas * 100) / 100
	local rpmK = carPh.rpm / 1000
	local boostMult = (1 + finalBoost) * (rpmK * (actualTrottle) + 2 * (1 - actualTrottle))
	local turboLagMult = 1 - math.saturate(turboLag)

	local trottleMult = math.max(actualTrottle, antilag * math.pow(antilagCounter, 0.5))

	local targetBoost = rpmLut:get(rpmK) * math.pow(trottleMult, 0.5) - (1 - math.pow(trottleMult, 0.5))

	finalBoost = finalBoost + (targetBoost - finalBoost) * boostMult * turboLagMult

	finalBoost = math.clamp(finalBoost, -0.7 + actualTrottle / 2, boostController)

	antilagCounter = actualTrottle < antilag and antilagCounter - antilagTime or antilagCounter + 0.01
	antilagCounter = math.saturate(antilagCounter) * math.saturate(rpmK - 1)

	ac.overrideTurboBoost(0, finalBoost, finalBoost)

	carPh.gas = carPh.gearUp and 0 or carPh.gas

end

return { update = update, reset = reset }
